/*
 * Copyright (C) 2010-2013 jeanfi@gmail.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <glib.h>
#include <json/json.h>

#include "http.h"
#include "io.h"
#include "log.h"
#include "ttrss_cache.h"
#include "ttrss_wsasync.h"
#include "url.h"

static struct feed **data;
char *ttrss_get_headline_content(struct headline *h)
{
	char *content;

	content = cache_get(h);

	if (content) {
		log_debug("ttrss_get_headline_content: cache hit");
	} else {
		log_debug("ttrss_get_headline_content: cache miss");
		content = ws_get_article_content(h->id);
		cache_put(h->id, content);
	}

	return content;
}

struct feed **ttrss_get_feeds()
{
	data = ws_update_feeds(data);

	return data;
}

struct headline **ttrss_feed_get_headlines(struct feed *f)
{
	if (!f->headlines)
		ws_update_headlines(f);

	return f->headlines;
}

void ttrss_set_article_unread(int id, int unread)
{
	log_debug("ttrss_set_article_unread(%d,%d)", id, unread);

	ws_async_set_article_unread(id, unread);

	log_debug("ttrss_set_article_unread(%d,%d)", id, unread);
}

void ttrss_set_config(const char *url, const char *user, const char *pwd)
{
	feeds_free(data);
	data = NULL;
	ws_set_config(url, user, pwd);
}

struct feed *ttrss_get_feed(int id)
{
	return feeds_get_feed(data, id);
}

struct headline *ttrss_get_headline(int id)
{
	return feeds_get_headline(data, id);
}

static void get_article_content_cbk(int id, const char *content)
{
	log_debug("get_article_content_cbk %d\n", id);

	if (content)
		cache_put(id, content);
}

void ttrs_download_headline_content(struct feed **feeds)
{
	struct feed **fcur;
	struct headline **hcur;

	log_debug("ttrs_download_headline_content");

	for (fcur = feeds; *fcur; fcur++) {
		hcur = ttrss_feed_get_headlines(*fcur);

		log_debug("ttrs_download_headline_content(): %s",
			  (*fcur)->title);

		while (hcur && *hcur) {
			if (!cache_exists(*hcur)) {
				ws_async_get_article_content
					((*hcur)->id, get_article_content_cbk);
			}
			hcur++;
		}
	}
}
